// Fragment Shader  file "refraction.frag"

#version 330

#define MAX_FOG			0.6

//uniform float transmittance;
uniform sampler2D texture0;
uniform samplerCube environmentMap;

//**********************************************************//
// Uniforms for fog effect
//**********************************************************//
uniform vec4  fog_color;
uniform float fog_density;

//**********************************************************//
// Inputs from the vertex shader
//**********************************************************//
in vec4 viewPos;			//position
in vec2 texCoord0;			//texture
in vec3 R;					//reflection

in vec3 tRed;
in vec3 tGreen;
in vec3 tBlue;
in float reflectionFactor;

out vec4 fragColor;	//final color of fragment

//**********************************************************//
// Fog generation
//**********************************************************//
vec4 generateFog(vec4 fragColor)	{
	const float LOG2 = 1.442695;

	float z = length(viewPos);
	
	float fogFactor = exp2(-fog_density * fog_density * z * z * LOG2);
	
	fogFactor = clamp(fogFactor, 0.0, 1.0);

	fogFactor = max(fogFactor, MAX_FOG);

	return mix(fog_color, fragColor, fogFactor);
}

//**********************************************************//
// Main entry point
//**********************************************************//
void main(void) {
	// Fetch the reflected environment color
	vec4 reflectedColor = texture(environmentMap, R);

	// Compute the refracted environment color
	vec4 refractedColor;
	refractedColor.r = texture(environmentMap, tRed).r;
	refractedColor.g = texture(environmentMap, tGreen).g;
	refractedColor.b = texture(environmentMap, tBlue).b;
	refractedColor.a = 1;

	// Compute the final color
	fragColor = mix(refractedColor, reflectedColor, 0.2f);
	fragColor = generateFog(fragColor);

	/*
	vec4 refractionColor = texture(environmentMap, R);
	//vec3 textureColor = texture(texture0, texCoord0.st).rgb;
	vec4 textureColor = vec4(1.0f, 1.0f, 1.0f, 1.0f);

	fragColor = mix(textureColor, refractionColor, 0.7f);
	//fragColor = generateFog(fragColor);
	*/
}