// Vertex Shader  file "refraction.vert"	

#version 330

uniform mat4 modelViewMatrix;	//model-view matrix
uniform mat4 projectionMatrix;	//projection matrix
uniform vec3 eyePositionWorld;	//eye position in world coordinates
uniform mat4 modelToWorld;		//model to world transformation matrix
uniform float refraction;

in vec4 vertex;		//local coordinates
in vec3 normal;		//local normals
in vec2 texure0;	//texture coordinates

//variables for fragment shader		
out vec4 viewPos;	//view position
out vec2 texCoord0;	//texture coordinates
out vec3 R;			//reflection

out vec3 tRed;
out vec3 tGreen;
out vec3 tBlue;
out float reflectionFactor;

void main(void) {
	//original
	viewPos = modelViewMatrix * vertex;
	texCoord0 = texure0;

	vec3 positionWorld = (modelToWorld * vertex).xyz;
	vec3 N = normalize(mat3(modelToWorld) * normal);

	vec3 V = positionWorld - eyePositionWorld;
	//R = refract(V, N, refraction);
	R = reflect(V, N);
	V = normalize(V);

	tRed = refract(V, N, 1.1f);
	tGreen = refract(V, N, 1.1f);
	tBlue = refract(V, N, 1.1f);

	float fresnelBias = 0.02f;
	float fresnelScale = 1;
	float fresnelPower = 4;

	reflectionFactor = fresnelBias + fresnelScale * pow(1 + dot(V, N), fresnelPower);

	gl_Position = projectionMatrix * viewPos;
}